/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: testascio.cpp 600 2018-08-22 16:14:36Z cag $
 * ---------------------------------------------------------------------
 *
 * Copyright (C)  Jan Snigula <snigula@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * ---------------------------------------------------------------------
 *
 */

#define LTL_RANGE_CHECKING

#include <ltl/marray.h>
#include <ltl/ascio.h>
#include <ltl/statistics.h>
#include <ltl/marray_io.h>
#include <iostream>
#include <fstream>

using namespace ltl;

int main(int argc, char **argv);

void testint( void )
{

   MArray<int,1> res1(7);
   res1 = 3, 4, 5, 6, 7, 8, 9;
   MArray<int,1> res1l1a(3);
   res1l1a = 3, 4, 5;
   MArray<int,1> res1l1b(2);
   res1l1b = 6, 7;
   MArray<int,1> res1l1c(2);
   res1l1c = 8, 9;
   MArray<int,1> res1l2a(4);
   res1l2a = 3, 4, 5, 6;
   MArray<int,1> res1l2b(5);
   res1l2b = 5, 6, 7, 8, 9;
   MArray<int,2> res2(3,7);
   res2 =
      3, 4, 5,
      4, 5, 6,
      5, 6, 7,
      6, 7, 8,
      7, 8, 9,
      8, 9, 0,
      9, 0, 1;
   MArray<int,2> res2l1a(3,3);
   res2l1a =
      3, 4, 5,
      4, 5, 6,
      5, 6, 7;
   MArray<int,2> res2l1b(3,2);
   res2l1b =
      6, 7, 8,
      7, 8, 9;
   MArray<int,2> res2l1c(3,2);
   res2l1c =
      8, 9, 0,
      9, 0, 1;
   MArray<int,2> res2l2a(3,4);
   res2l2a =
      3, 4, 5,
      4, 5, 6,
      5, 6, 7,
      6, 7, 8;
   MArray<int,2> res2l2b(3,5);
   res2l2b =
      5, 6, 7,
      6, 7, 8,
      7, 8, 9,
      8, 9, 0,
      9, 0, 1;

   std::cerr << std::endl << "Testing integer data file" << std::endl;
   std::cerr << "Creating testfile...";
   std::ofstream file("test.dat");

   file << "1 2 3 4 5" << std::endl
	<< " # ugly comment" << std::endl
	<< "2 3 4 5 6" << std::endl
	<< "  3  4 5 6 7" << std::endl
	<< "4 5 6 07 8" << std::endl
	<< "#comment" << std::endl
	<< "5   6 7 8 9" << std::endl
	<< "6 7 8 9 0" << std::endl
	<< "7  8 9 0 1" << std::endl;
   file.close();

   std::cerr << "done" << std::endl;

   AscFile f( "test.dat" );

   std::cerr << "Testing rows()...";

   if( f.rows() != 7 ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing cols()...";

   if( f.cols() != 5 ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumn()...";

   MArray<int,1> read1 = f.readIntColumn(3);

   if( ! allof( read1 == res1 ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumns()...";

   MArray<int,2> read2 = f.readIntColumns(3,5);

   if( ! allof( read2 == res2 ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;


   std::cerr << "Testing readIntColumn() with lineranges...";

   MArray<int,1> read1l1a = f.readIntColumn(3,1,3);

   if( ! allof( read1l1a == res1l1a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumn() with lineranges...";

   MArray<int,1> read1l1b = f.readIntColumn(3,4,2);

   if( ! allof( read1l1b == res1l1b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumn() with lineranges...";

   MArray<int,1> read1l1c = f.readIntColumn(3,6,2);

   if( ! allof( read1l1c == res1l1c ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumn() with lineranges...";

   MArray<int,1> read1l2a = f.readIntColumn(3,1,4);

   if( ! allof( read1l2a == res1l2a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumn() with lineranges...";

   MArray<int,1> read1l2b = f.readIntColumn(3,3,5);

   if( ! allof( read1l2b == res1l2b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;



   std::cerr << "Testing readIntColumns() with lineranges...";

   MArray<int,2> read2l1a = f.readIntColumns(3,5,1,3);

   if( ! allof( read2l1a == res2l1a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumns() with lineranges...";

   MArray<int,2> read2l1b = f.readIntColumns(3,5,4,2);

   if( ! allof( read2l1b == res2l1b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumns() with lineranges...";

   MArray<int,2> read2l1c = f.readIntColumns(3,5,6,2);

   if( ! allof( read2l1c == res2l1c ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumns() with lineranges...";

   MArray<int,2> read2l2a = f.readIntColumns(3,5,1,4);

   if( ! allof( read2l2a == res2l2a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumns() with lineranges...";

   MArray<int,2> read2l2b = f.readIntColumns(3,5,3,5);

   if( ! allof( read2l2b == res2l2b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

}

void testfloat( void )
{

   MArray<float,1> res1(7);
   res1 = 3.2, 4.3, 5.4, 6.5, 7.6, 8.7, 9.8;
   MArray<float,1> res1l1a(3);
   res1l1a = 3.2, 4.3, 5.4;
   MArray<float,1> res1l1b(2);
   res1l1b = 6.5, 7.6;
   MArray<float,1> res1l1c(2);
   res1l1c = 8.7, 9.8;
   MArray<float,1> res1l2a(4);
   res1l2a = 3.2, 4.3, 5.4, 6.5;
   MArray<float,1> res1l2b(5);
   res1l2b = 5.4, 6.5, 7.6, 8.7, 9.8;
   MArray<float,2> res2(3,7);
   res2 =
      3.2, 4.3, 5.4,
      4.3, 5.4, 6.5,
      5.4, 6.5, 7.6,
      6.5, 7.6, 8.7,
      7.6, 8.7, 9.8,
      8.7, 9.8, 0.9,
      9.8, 0.9, 1.0;
   MArray<float,2> res2l1a(3,3);
   res2l1a =
      3.2, 4.3, 5.4,
      4.3, 5.4, 6.5,
      5.4, 6.5, 7.6;
   MArray<float,2> res2l1b(3,2);
   res2l1b =
      6.5, 7.6, 8.7,
      7.6, 8.7, 9.8;
   MArray<float,2> res2l1c(3,2);
   res2l1c =
      8.7, 9.8, 0.9,
      9.8, 0.9, 1.0;
   MArray<float,2> res2l2a(3,4);
   res2l2a =
      3.2, 4.3, 5.4,
      4.3, 5.4, 6.5,
      5.4, 6.5, 7.6,
      6.5, 7.6, 8.7;
   MArray<float,2> res2l2b(3,5);
   res2l2b =
      5.4, 6.5, 7.6,
      6.5, 7.6, 8.7,
      7.6, 8.7, 9.8,
      8.7, 9.8, 0.9,
      9.8, 0.9, 1.0;

   std::cerr << std::endl << "Testing float data file" << std::endl;
   std::cerr << "Creating testfile...";
   std::ofstream file("test.dat");

   file << "1.0 2.1 3.20 4.3 5.4" << std::endl
	<< " # ugly comment" << std::endl
	<< "2.100000 3.2 4.3 5.40 6.5" << std::endl
	<< "  3.200  04.30 5.4 6.5 7.6" << std::endl
	<< "4.3 5.4 6.50 7.6 8.7" << std::endl
	<< "#comment" << std::endl
	<< "5.40000000000000000000000000000   6.5 7.60 8.70 9.8" << std::endl
	<< "6.5 7.6 8.7 9.8 0.9000000000000000000000000000000000000000" << std::endl
	<< "7.6  8.7000 9.80 0.90 1.00" << std::endl;
   file.close();

   std::cerr << "done" << std::endl;

   AscFile f( "test.dat" );

   std::cerr << "Testing rows()...";

   if( f.rows() != 7 ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing cols()...";

   if( f.cols() != 5 ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumn()...";

   MArray<float,1> read1 = f.readFloatColumn(3);
   if( ! allof( read1 == res1 ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumns()...";

   MArray<float,2> read2 = f.readFloatColumns(3,5);

   if( ! allof( read2 == res2 ) ) {
      std::cerr << read2 << std::endl;
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;


   std::cerr << "Testing readFloatColumn() with lineranges...";

   MArray<float,1> read1l1a = f.readFloatColumn(3,1,3);

   if( ! allof( read1l1a == res1l1a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumn() with lineranges...";

   MArray<float,1> read1l1b = f.readFloatColumn(3,4,2);

   if( ! allof( read1l1b == res1l1b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumn() with lineranges...";

   MArray<float,1> read1l1c = f.readFloatColumn(3,6,2);

   if( ! allof( read1l1c == res1l1c ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumn() with lineranges...";

   MArray<float,1> read1l2a = f.readFloatColumn(3,1,4);

   if( ! allof( read1l2a == res1l2a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumn() with lineranges...";

   MArray<float,1> read1l2b = f.readFloatColumn(3,3,5);

   if( ! allof( read1l2b == res1l2b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;



   std::cerr << "Testing readFloatColumns() with lineranges...";

   MArray<float,2> read2l1a = f.readFloatColumns(3,5,1,3);

   if( ! allof( read2l1a == res2l1a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumns() with lineranges...";

   MArray<float,2> read2l1b = f.readFloatColumns(3,5,4,2);

   if( ! allof( read2l1b == res2l1b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumns() with lineranges...";

   MArray<float,2> read2l1c = f.readFloatColumns(3,5,6,2);

   if( ! allof( read2l1c == res2l1c ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumns() with lineranges...";

   MArray<float,2> read2l2a = f.readFloatColumns(3,5,1,4);

   if( ! allof( read2l2a == res2l2a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readFloatColumns() with lineranges...";

   MArray<float,2> read2l2b = f.readFloatColumns(3,5,3,5);

   if( ! allof( read2l2b == res2l2b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;




   MArray<float,2> res3(2,7);
   res3 =
      3.2, 5.4,
      4.3, 6.5,
      5.4, 7.6,
      6.5, 8.7,
      7.6, 9.8,
      8.7, 0.9,
      9.8, 1.0;

   int c[2] = {3,5};
   MArray<float,2> read3 = f.readFloatColumns(c,2);

   if( ! allof( read3 == res3 ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;
}

void teststring( void )
{

   MArray<std::string,1> res1(7);
   res1 = "c", "d", "e", "f", "g", "h", "i";
   MArray<std::string,1> res1l1a(3);
   res1l1a = "c", "d", "e";
   MArray<std::string,1> res1l1b(2);
   res1l1b = "f", "g";
   MArray<std::string,1> res1l1c(2);
   res1l1c = "h", "i";
   MArray<std::string,1> res1l2a(4);
   res1l2a = "c", "d", "e", "f";
   MArray<std::string,1> res1l2b(5);
   res1l2b = "e", "f", "g", "h", "i";
   MArray<std::string,2> res2(3,7);
   res2 =
      "c", "d", "e",
      "d", "e", "f",
      "e", "f", "g",
      "f", "gg", "h",
      "g", "h", "i",
      "h", "i", "j",
      "i", "j", "k";
   MArray<std::string,2> res2l1a(3,3);
   res2l1a =
      "c", "d", "e",
      "d", "e", "f",
      "e", "f", "g";
   MArray<std::string,2> res2l1b(3,2);
   res2l1b =
      "f", "gg", "h",
      "g", "h", "i";
   MArray<std::string,2> res2l1c(3,2);
   res2l1c =
      "h", "i", "j",
      "i", "j", "k";
   MArray<std::string,2> res2l2a(3,4);
   res2l2a =
      "c", "d", "e",
      "d", "e", "f",
      "e", "f", "g",
      "f", "gg", "h";
   MArray<std::string,2> res2l2b(3,5);
   res2l2b =
      "e", "f", "g",
      "f", "gg", "h",
      "g", "h", "i",
      "h", "i", "j",
      "i", "j", "k";

   std::cerr << std::endl << "Testing string data file" << std::endl;
   std::cerr << "Creating testfile...";
   std::ofstream file("test.dat");

   file << "a b c d e" << std::endl
	<< " # ugly comment" << std::endl
	<< "b c d e f" << std::endl
	<< "  c  d e f g" << std::endl
	<< "d e f gg h" << std::endl
	<< "#comment" << std::endl
	<< "e   f g h i" << std::endl
	<< "f g h i j" << std::endl
	<< "g  h i j k" << std::endl;
   file.close();

   std::cerr << "done" << std::endl;

   AscFile f( "test.dat" );

   std::cerr << "Testing rows()...";

   if( f.rows() != 7 ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing cols()...";

   if( f.cols() != 5 ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumn()...";

   MArray<std::string,1> read1 = f.readStringColumn(3);

   if( ! allof( read1 == res1 ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumns()...";

   MArray<std::string,2> read2 = f.readStringColumns(3,5);

   if( ! allof( read2 == res2 ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;


   std::cerr << "Testing readStringColumn() with lineranges...";

   MArray<std::string,1> read1l1a = f.readStringColumn(3,1,3);

   if( ! allof( read1l1a == res1l1a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumn() with lineranges...";

   MArray<std::string,1> read1l1b = f.readStringColumn(3,4,2);

   if( ! allof( read1l1b == res1l1b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumn() with lineranges...";

   MArray<std::string,1> read1l1c = f.readStringColumn(3,6,2);

   if( ! allof( read1l1c == res1l1c ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumn() with lineranges...";

   MArray<std::string,1> read1l2a = f.readStringColumn(3,1,4);

   if( ! allof( read1l2a == res1l2a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumn() with lineranges...";

   MArray<std::string,1> read1l2b = f.readStringColumn(3,3,5);

   if( ! allof( read1l2b == res1l2b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumns() with lineranges...";

   MArray<std::string,2> read2l1a = f.readStringColumns(3,5,1,3);

   if( ! allof( read2l1a == res2l1a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumns() with lineranges...";

   MArray<std::string,2> read2l1b = f.readStringColumns(3,5,4,2);

   if( ! allof( read2l1b == res2l1b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumns() with lineranges...";

   MArray<std::string,2> read2l1c = f.readStringColumns(3,5,6,2);

   if( ! allof( read2l1c == res2l1c ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readIntColumns() with lineranges...";

   MArray<std::string,2> read2l2a = f.readStringColumns(3,5,1,4);

   if( ! allof( read2l2a == res2l2a ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

   std::cerr << "Testing readStringColumns() with lineranges...";

   MArray<std::string,2> read2l2b = f.readStringColumns(3,5,3,5);

   if( ! allof( read2l2b == res2l2b ) ) {
      std::cerr << "failed" << std::endl;
      exit(-1);
   } else
      std::cerr << "passed" << std::endl;

}

void testheader( void )
{

   std::cerr << std::endl << "Testing getHeader with normal comments" << std::endl;
   std::cerr << "Creating testfile...";
   std::ofstream file("test.dat");

   file << "# Comment line 1 " << std::endl
	<< "  # Comment line 2 " << std::endl
	<< "# Comment line 3 " << std::endl
	<< "   " << std::endl
	<< "# Comment line 4 " << std::endl
	<< "" << std::endl
	<< "        # Comment line 5 " << std::endl
	<< "# Comment line 6 " << std::endl
	<< "1.0 2.1 3.20 4.3 5.4" << std::endl
	<< " # ugly comment" << std::endl
	<< "2.100000 3.2 4.3 5.40 6.5" << std::endl
	<< "  3.200  04.30 5.4 6.5 7.6" << std::endl
	<< "4.3 5.4 6.50 7.6 8.7" << std::endl
	<< "#comment" << std::endl
	<< "5.40000000000000000000000000000   6.5 7.60 8.70 9.8" << std::endl
	<< "6.5 7.6 8.7 9.8 0.9000000000000000000000000000000000000000" << std::endl
	<< "7.6  8.7000 9.80 0.90 1.00" << std::endl;
   file.close();

   std::cerr << "done" << std::endl;

   AscFile f( "test.dat" );

   std::cerr << "Testing with skipped comment signs...";

   std::vector<std::string> v1;

   f.getHeader( v1, false );

   if( v1[0] != " Comment line 1 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 1 \" got \"" << v1[0] << "\"" << std::endl;
   else if( v1[1] != " Comment line 2 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 2 \" got \"" << v1[1] << "\"" << std::endl;
   else if( v1[2] != " Comment line 3 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 3 \" got \"" << v1[2] << "\"" << std::endl;
   else if( v1[3] != " Comment line 4 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 4 \" got \"" << v1[3] << "\"" << std::endl;
   else if( v1[4] != " Comment line 5 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 5 \" got \"" << v1[4] << "\"" << std::endl;
   else if( v1[5] != " Comment line 6 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 6 \" got \"" << v1[5] << "\"" << std::endl;
   else
      std::cerr << "passed" << std::endl;


   std::cerr << "Testing without skipped comment signs...";

   v1.clear();

   f.getHeader( v1, true );

   if( v1[0] != "# Comment line 1 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"# Comment line 1 \" got \"" << v1[0] << "\"" << std::endl;
   else if( v1[1] != "# Comment line 2 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"# Comment line 2 \" got \"" << v1[1] << "\"" << std::endl;
   else if( v1[2] != "# Comment line 3 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"# Comment line 3 \" got \"" << v1[2] << "\"" << std::endl;
   else if( v1[3] != "# Comment line 4 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"# Comment line 4 \" got \"" << v1[3] << "\"" << std::endl;
   else if( v1[4] != "# Comment line 5 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"# Comment line 5 \" got \"" << v1[4] << "\"" << std::endl;
   else if( v1[5] != "# Comment line 6 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"# Comment line 6 \" got \"" << v1[5] << "\"" << std::endl;
   else
      std::cerr << "passed" << std::endl;


   std::cerr << std::endl << "Testing getHeader with weird comments" << std::endl;
   std::cerr << "Creating testfile...";
   std::ofstream file2("test.dat");

   file2 << "COMMENT Comment line 1 " << std::endl
	 << "  COMMENT Comment line 2 " << std::endl
	 << "COMMENT Comment line 3 " << std::endl
	 << "   " << std::endl
	 << "COMMENT Comment line 4 " << std::endl
	 << "" << std::endl
	 << "        COMMENT Comment line 5 " << std::endl
	 << "COMMENT Comment line 6 " << std::endl
	 << "1.0 2.1 3.20 4.3 5.4" << std::endl
	 << " COMMENT ugly comment" << std::endl
	 << "2.100000 3.2 4.3 5.40 6.5" << std::endl
	 << "  3.200  04.30 5.4 6.5 7.6" << std::endl
	 << "4.3 5.4 6.50 7.6 8.7" << std::endl
	 << "COMMENTcomment" << std::endl
	 << "5.40000000000000000000000000000   6.5 7.60 8.70 9.8" << std::endl
	 << "6.5 7.6 8.7 9.8 0.9000000000000000000000000000000000000000" << std::endl
	 << "7.6  8.7000 9.80 0.90 1.00" << std::endl;
   file.close();

   std::cerr << "done" << std::endl;

   AscFile f2( "test.dat", 0, "COMMENT" );

   std::cerr << "Testing with skipped comment signs...";

   v1.clear();

   f2.getHeader( v1, false );

   if( v1[0] != " Comment line 1 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 1 \" got \"" << v1[0] << "\"" << std::endl;
   else if( v1[1] != " Comment line 2 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 2 \" got \"" << v1[1] << "\"" << std::endl;
   else if( v1[2] != " Comment line 3 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 3 \" got \"" << v1[2] << "\"" << std::endl;
   else if( v1[3] != " Comment line 4 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 4 \" got \"" << v1[3] << "\"" << std::endl;
   else if( v1[4] != " Comment line 5 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 5 \" got \"" << v1[4] << "\"" << std::endl;
   else if( v1[5] != " Comment line 6 " )
      std::cerr << "failed" << std::endl
		<< "Expected \" Comment line 6 \" got \"" << v1[5] << "\"" << std::endl;
   else
      std::cerr << "passed" << std::endl;


   std::cerr << "Testing without skipped comment signs...";

   v1.clear();

   f2.getHeader( v1, true );

   if( v1[0] != "COMMENT Comment line 1 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"COMMENT Comment line 1 \" got \"" << v1[0] << "\"" << std::endl;
   else if( v1[1] != "COMMENT Comment line 2 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"COMMENT Comment line 2 \" got \"" << v1[1] << "\"" << std::endl;
   else if( v1[2] != "COMMENT Comment line 3 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"COMMENT Comment line 3 \" got \"" << v1[2] << "\"" << std::endl;
   else if( v1[3] != "COMMENT Comment line 4 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"COMMENT Comment line 4 \" got \"" << v1[3] << "\"" << std::endl;
   else if( v1[4] != "COMMENT Comment line 5 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"COMMENT Comment line 5 \" got \"" << v1[4] << "\"" << std::endl;
   else if( v1[5] != "COMMENT Comment line 6 " )
      std::cerr << "failed" << std::endl
		<< "Expected \"COMMENT Comment line 6 \" got \"" << v1[5] << "\"" << std::endl;
   else
      std::cerr << "passed" << std::endl;

}

int main(int, char **)
{
   std::cerr << "Testing MArray ASCII I/O ..." << std::endl;
   testint();
   testfloat();
   teststring();
   testheader();
}
