/* -*- C++ -*-
 *
 * ---------------------------------------------------------------------
 * $Id: testutildate.cpp 600 2018-08-22 16:14:36Z cag $
 * ---------------------------------------------------------------------
 *
 * Copyright (C)  Niv Drory <drory@mpe.mpg.de>
 *                         Claus A. Goessl <cag@usm.uni-muenchen.de>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA
 *
 * ---------------------------------------------------------------------
 *
 */


#include <ltl/util/utdate.h>
#include <iostream>

using namespace util;

int main()
{
   try
   {      
      std::cerr << "Testing util date methods:" << std::endl;

      std::cerr << "Now:" << std::endl;

      UTDate cnow;
      DCFDate dcfnow(cnow);
      FitsDate fitsnow(cnow);
      JulDate julnow(cnow);
   
      std::cerr << "ctime:    " << cnow.toString() << std::endl;
      std::cerr << "DCFDate:  " << dcfnow.toString() << std::endl;
      std::cerr << "FITSDate: " << fitsnow.toString() << std::endl;
      std::cerr << "JulDate:  " << julnow.toString() << std::endl;

      std::cerr << "Epoch:" << std::endl;

      UTDate cepoch( time_t(0) );
      DCFDate dcfepoch(cepoch);
      FitsDate fitsepoch(cepoch);
      JulDate julepoch(cepoch);
   
      std::cerr << "ctime:    " << cepoch.toString() << std::endl;
      std::cerr << "DCFDate:  " << dcfepoch.toString() << std::endl;
      std::cerr << "FITSDate: " << fitsepoch.toString() << std::endl;
      std::cerr << "JulDate:  " << julepoch.toString() << std::endl;

      std::cerr << "String Init for 4th March 1971, 18:10:00 UT" << std::endl;
   
      const std::string dcfbirth("D:04.03.71;T:6;U:19.10.00;    ");
      
      const std::string fitsbirthday("04/03/71");
      const std::string fitsbirth("1971-03-04T18:10:00");
      const double julbirth(2441015.25694444);
      std::cerr << "DCFDate:  " << UTDate(DCFDate(dcfbirth)).toString() 
		<< " out of " << dcfbirth << std::endl;
      std::cerr << "FITSDate: " << UTDate(FitsDate(fitsbirth)).toString() 
		<< " out of " << fitsbirth << std::endl;
      std::cerr << "day only: " << UTDate(FitsDate(fitsbirthday)).toString()
		<< " out of " << fitsbirthday << std::endl;
      std::cerr << "JulDate:  " << UTDate(JulDate(julbirth)).toString()
		<< " out of " << julbirth << std::endl;
   }
   catch(UTDateException ue)
   {
      std::cerr << ue.what() << std::endl;
      exit(1);
   }
   
   return 0;
}
